@@ -249,7 +249,7 @@ int bch2_alloc_v4_validate(struct bch_fs *c, struct bkey_s_c k,
 	struct bch_alloc_v4 a;
 	int ret = 0;
 
-	bkey_val_copy(&a, bkey_s_c_to_alloc_v4(k));
+	bkey_val_copy_pad(&a, bkey_s_c_to_alloc_v4(k));
 
 	bkey_fsck_err_on(alloc_v4_u64s_noerror(&a) > bkey_val_u64s(k.k),
 			 c, alloc_v4_val_size_bad,
@@ -325,7 +325,7 @@ fsck_err:
 	return ret;
 }
 
-void bch2_alloc_v4_swab(struct bkey_s k)
+void bch2_alloc_v4_swab(const struct bch_fs *c, struct bkey_s k)
 {
 	struct bch_alloc_v4 *a = bkey_s_to_alloc_v4(k).v;
 
@@ -364,7 +364,6 @@ static inline void __bch2_alloc_v4_to_text(struct printbuf *out, struct bch_fs *
 		prt_printf(out, "stripe_sectors       %u\n",	a->stripe_sectors);
 	prt_printf(out, "cached_sectors       %u\n",	a->cached_sectors);
 	prt_printf(out, "stripe               %u\n",	a->stripe);
-	prt_printf(out, "stripe_redundancy    %u\n",	a->stripe_redundancy);
 	prt_printf(out, "io_time[READ]        %llu\n",	a->io_time[READ]);
 	prt_printf(out, "io_time[WRITE]       %llu\n",	a->io_time[WRITE]);
 
@@ -393,7 +392,7 @@ void __bch2_alloc_to_v4(struct bkey_s_c k, struct bch_alloc_v4 *out)
 	if (k.k->type == KEY_TYPE_alloc_v4) {
 		void *src, *dst;
 
-		bkey_val_copy(out, bkey_s_c_to_alloc_v4(k));
+		bkey_val_copy_pad(out, bkey_s_c_to_alloc_v4(k));
 
 		src = alloc_v4_backpointers(out);
 		SET_BCH_ALLOC_V4_BACKPOINTERS_START(out, BCH_ALLOC_V4_U64s);
@@ -412,7 +411,6 @@ void __bch2_alloc_to_v4(struct bkey_s_c k, struct bch_alloc_v4 *out)
 			.gen			= u.gen,
 			.oldest_gen		= u.oldest_gen,
 			.data_type		= u.data_type,
-			.stripe_redundancy	= u.stripe_redundancy,
 			.dirty_sectors		= u.dirty_sectors,
 			.cached_sectors		= u.cached_sectors,
 			.io_time[READ]		= u.read_time,
@@ -483,16 +481,9 @@ bch2_trans_start_alloc_update_noupdate(struct btree_trans *trans, struct btree_i
 	struct bkey_s_c k = bch2_btree_iter_peek_slot(iter);
 	int ret = bkey_err(k);
 	if (unlikely(ret))
-		goto err;
+		return ERR_PTR(ret);
 
-	struct bkey_i_alloc_v4 *a = bch2_alloc_to_v4_mut_inlined(trans, k);
-	ret = PTR_ERR_OR_ZERO(a);
-	if (unlikely(ret))
-		goto err;
-	return a;
-err:
-	bch2_trans_iter_exit(iter);
-	return ERR_PTR(ret);
+	return bch2_alloc_to_v4_mut_inlined(trans, k);
 }
 
 __flatten
@@ -1234,7 +1225,7 @@ put_ref:
 static int invalidate_one_bp(struct btree_trans *trans,
 			     struct bch_dev *ca,
 			     struct bkey_s_c_backpointer bp,
-			     struct bkey_buf *last_flushed)
+			     struct wb_maybe_flush *last_flushed)
 {
 	CLASS(btree_iter_uninit, iter)(trans);
 	struct bkey_s_c k = bkey_try(bch2_backpointer_get_key(trans, bp, &iter, 0, last_flushed));
@@ -1244,7 +1235,7 @@ static int invalidate_one_bp(struct btree_trans *trans,
 	struct bkey_i *n = errptr_try(bch2_bkey_make_mut(trans, &iter, &k,
 						BTREE_UPDATE_internal_snapshot_node));
 
-	bch2_bkey_drop_device(bkey_i_to_s(n), ca->dev_idx);
+	bch2_bkey_drop_device(trans->c, bkey_i_to_s(n), ca->dev_idx);
 	return 0;
 }
 
@@ -1252,7 +1243,7 @@ static int invalidate_one_bucket_by_bps(struct btree_trans *trans,
 					struct bch_dev *ca,
 					struct bpos bucket,
 					u8 gen,
-					struct bkey_buf *last_flushed)
+					struct wb_maybe_flush *last_flushed)
 {
 	struct bpos bp_start	= bucket_pos_to_bp_start(ca,	bucket);
 	struct bpos bp_end	= bucket_pos_to_bp_end(ca,	bucket);
@@ -1281,7 +1272,7 @@ static int invalidate_one_bucket(struct btree_trans *trans,
 				 struct bch_dev *ca,
 				 struct btree_iter *lru_iter,
 				 struct bkey_s_c lru_k,
-				 struct bkey_buf *last_flushed,
+				 struct wb_maybe_flush *last_flushed,
 				 s64 *nr_to_invalidate)
 {
 	struct bch_fs *c = trans->c;
@@ -1357,35 +1348,30 @@ again:
 	return k;
 }
 
-static void bch2_do_invalidates_work(struct work_struct *work)
+static void __bch2_do_invalidates(struct bch_dev *ca)
 {
-	struct bch_dev *ca = container_of(work, struct bch_dev, invalidate_work);
 	struct bch_fs *c = ca->fs;
 	CLASS(btree_trans, trans)(c);
-	int ret = 0;
 
-	struct bkey_buf last_flushed __cleanup(bch2_bkey_buf_exit);
-	bch2_bkey_buf_init(&last_flushed);
+	struct wb_maybe_flush last_flushed __cleanup(wb_maybe_flush_exit);
+	wb_maybe_flush_init(&last_flushed);
 
-	ret = bch2_btree_write_buffer_tryflush(trans);
-	if (ret)
-		goto err;
+	bch2_btree_write_buffer_tryflush(trans);
 
 	s64 nr_to_invalidate =
 		should_invalidate_buckets(ca, bch2_dev_usage_read(ca));
-	struct btree_iter iter;
 	bool wrapped = false;
 
-	bch2_trans_iter_init(trans, &iter, BTREE_ID_lru,
-			     lru_pos(ca->dev_idx, 0,
-				     ((bch2_current_io_time(c, READ) + U32_MAX) &
-				      LRU_TIME_MAX)), 0);
+	CLASS(btree_iter, iter)(trans, BTREE_ID_lru,
+				lru_pos(ca->dev_idx, 0,
+					((bch2_current_io_time(c, READ) + U32_MAX) &
+					 LRU_TIME_MAX)), 0);
 
 	while (true) {
 		bch2_trans_begin(trans);
 
 		struct bkey_s_c k = next_lru_key(trans, &iter, ca, &wrapped);
-		ret = bkey_err(k);
+		int ret = bkey_err(k);
 		if (ret)
 			goto restart_err;
 		if (!k.k)
@@ -1398,10 +1384,18 @@ restart_err:
 		if (ret)
 			break;
 
+		wb_maybe_flush_inc(&last_flushed);
 		bch2_btree_iter_advance(&iter);
 	}
-	bch2_trans_iter_exit(&iter);
-err:
+}
+
+static void bch2_do_invalidates_work(struct work_struct *work)
+{
+	struct bch_dev *ca = container_of(work, struct bch_dev, invalidate_work);
+	struct bch_fs *c = ca->fs;
+
+	__bch2_do_invalidates(ca);
+
 	enumerated_ref_put(&ca->io_ref[WRITE], BCH_DEV_WRITE_REF_do_invalidates);
 	enumerated_ref_put(&c->writes, BCH_WRITE_REF_invalidate);
 }
@@ -1444,15 +1438,15 @@ int bch2_dev_remove_alloc(struct bch_fs *c, struct bch_dev *ca)
 	 */
 	ret =   bch2_dev_remove_lrus(c, ca) ?:
 		bch2_btree_delete_range(c, BTREE_ID_need_discard, start, end,
-					BTREE_TRIGGER_norun, NULL) ?:
+					BTREE_TRIGGER_norun) ?:
 		bch2_btree_delete_range(c, BTREE_ID_freespace, start, end,
-					BTREE_TRIGGER_norun, NULL) ?:
+					BTREE_TRIGGER_norun) ?:
 		bch2_btree_delete_range(c, BTREE_ID_backpointers, start, end,
-					BTREE_TRIGGER_norun, NULL) ?:
+					BTREE_TRIGGER_norun) ?:
 		bch2_btree_delete_range(c, BTREE_ID_bucket_gens, start, end,
-					BTREE_TRIGGER_norun, NULL) ?:
+					BTREE_TRIGGER_norun) ?:
 		bch2_btree_delete_range(c, BTREE_ID_alloc, start, end,
-					BTREE_TRIGGER_norun, NULL) ?:
+					BTREE_TRIGGER_norun) ?:
 		bch2_dev_usage_remove(c, ca);
 	bch_err_msg(ca, ret, "removing dev alloc info");
 	return ret;
@@ -1593,13 +1587,22 @@ void bch2_dev_allocator_set_rw(struct bch_fs *c, struct bch_dev *ca, bool rw)
 {
 	/* BCH_DATA_free == all rw devs */
 
-	for (unsigned i = 0; i < ARRAY_SIZE(c->rw_devs); i++)
-		if (rw &&
-		    (i == BCH_DATA_free ||
-		     (ca->mi.data_allowed & BIT(i))))
-			set_bit(ca->dev_idx, c->rw_devs[i].d);
-		else
-			clear_bit(ca->dev_idx, c->rw_devs[i].d);
+	for (unsigned i = 0; i < ARRAY_SIZE(c->rw_devs); i++) {
+		bool data_type_rw = rw;
+
+		if (i != BCH_DATA_free &&
+		    !(ca->mi.data_allowed & BIT(i)))
+			data_type_rw = false;
+
+		if ((i == BCH_DATA_journal ||
+		     i == BCH_DATA_btree) &&
+		    !ca->mi.durability)
+			data_type_rw = false;
+
+		mod_bit(ca->dev_idx, c->rw_devs[i].d, data_type_rw);
+	}
+
+	c->rw_devs_change_count++;
 }
 
 /* device goes ro: */
@@ -1610,8 +1613,6 @@ void bch2_dev_allocator_remove(struct bch_fs *c, struct bch_dev *ca)
 	/* First, remove device from allocation groups: */
 	bch2_dev_allocator_set_rw(c, ca, false);
 
-	c->rw_devs_change_count++;
-
 	/*
 	 * Capacity is calculated based off of devices in allocation groups:
 	 */
